@@ -30,11 +30,11 @@
 
 _CCCL_BEGIN_NAMESPACE_CUDA
 
-struct grid_level : __native_hierarchy_level_base<grid_level>
+struct _CCCL_DECLSPEC_EMPTY_BASES grid_level : __native_hierarchy_level_base<grid_level>
 {
-  using product_type  = unsigned long long;
-  using allowed_above = allowed_levels<>;
-  using allowed_below = allowed_levels<block_level, cluster_level>;
+  using __product_type  = unsigned long long;
+  using __allowed_above = __allowed_levels<>;
+  using __allowed_below = __allowed_levels<block_level, cluster_level>;
 };
 
 _CCCL_GLOBAL_CONSTANT grid_level grid;
